﻿#!/usr/bin/python
# coding: UTF-8
#可視レイヤーをタイル状に並べる


from gimpfu import *
import math

def tile(img, draw=None, num_tiles=1, resize=False, resize_width=400, add_title=False, title_height=40, font_name="Sans", font_size=30, delete_text=False):
 pdb.gimp_undo_push_group_start(img)
 if delete_text: # テキストレイヤーを削除して再作成
  for l in img.layers:
   if pdb.gimp_item_is_text_layer(l):
    pdb.gimp_image_remove_layer(img, l)
 layers=[x for x in img.layers if x.visible and not pdb.gimp_item_is_text_layer(x) and not pdb.gimp_item_is_group(x)] #可視レイヤーかつテキストレイヤーでないかつグループでない
 if not layers:
  pdb.gimp_undo_push_group_end(img)
  return
 max_width=-1
 max_height=-1
 for l in layers: # 前処理、最大幅、最大高さを探す
  if resize: # 画像の大きさを変更（変更してから最大幅を探す）
   resize_height=resize_width * l.height / l.width
   pdb.gimp_layer_scale(l, resize_width, resize_height, False)
  if l.width > max_width: max_width = l.width
  if l.height > max_height: max_height = l.height
 if add_title: # タイトルを追加するためのテキストレイヤーを追加する場合
  max_height = max_height + title_height
 num_layers=len(layers)
 rows=int(math.ceil(1.0*num_layers/num_tiles))
 pdb.gimp_image_resize(img, max_width * num_tiles, max_height * rows, 0, 0) # キャンパスサイズを変更
 for i in range(num_layers):
  ox=layers[i].offsets[0]
  oy=layers[i].offsets[1]
  pdb.gimp_layer_translate(layers[i], max_width * (i%num_tiles) - ox, max_height * (int(i/num_tiles)) - oy) #レイヤーを移動
  if add_title: # タイトルを追加するためのテキストレイヤーを追加する場合
   name = layers[i].name
   text_lay = pdb.gimp_text_layer_new(img, name, font_name, font_size, 0)
   img.add_layer(text_lay, pdb.gimp_image_get_layer_position(img, layers[i]))
   pdb.gimp_layer_translate(text_lay, max_width * (i%num_tiles), max_height * (int(i/num_tiles) + 1) - title_height)
   pdb.gimp_text_layer_set_justification(text_lay, TEXT_JUSTIFY_CENTER)
   pdb.gimp_text_layer_resize(text_lay, max_width, title_height)
 pdb.gimp_undo_push_group_end(img)


register(
        "python_fu_tile_layers",
        "可視レイヤーをタイル状に並べる",
        "tile layers",
        "amekujira",
        "amekujira",
        "2015",
        "<Image>/Layer/可視レイヤーをタイル状に並べる",
        "*",
        [
        (PF_SPINNER, "num_tiles", "１列あたりのタイル数", 2, (1, 64, 1)),
        (PF_BOOL,   "resize", "画像の大きさを変更", False),
        (PF_SPINNER, "resize_width", "変更する場合、画像の横幅", 400, (5, 1024, 1)),
        (PF_BOOL,   "add_title", "タイトルを追加", False),
        (PF_SPINNER, "title_height", "タイトルの高さ", 40, (0, 400, 1)),
        (PF_FONT, "font_name", "フォント", "Sans"),
        (PF_SPINNER, "font_size", "フォントの大きさ", 30, (0, 300, 1)),
        (PF_BOOL,   "delete_text", "テキストレイヤーをすべて消去して再作成", False),
	],
        [],
        tile)

main()
